Dialog = Object:extend()

local yoffset = 133
local lineheight = 10
Dialog.speechspeed = 1

function Dialog:new(orig, origcolor, lines)
  self.origcolor = origcolor
  self.lines = lines
  self.mx = 0
  self.my = 0
  self.t = 0
  if type(orig) == "string" then
    self.line = orig
    self.orig = {}
  else
    self.line = orig[1]
    self.orig = orig
    table.remove(orig, 1)
  end
  self.nextT = self.t + (#self.line * 0.04 + 3) / Dialog.speechspeed
  self.showAnswers = false
end


function Dialog:update(dt)
  self.t = self.t + dt

  if self.t > self.nextT then
    if #self.orig > 0 then
      self.line = self.orig[1]
      table.remove(self.orig, 1)
      self.nextT = self.t + (#self.line * 0.04 + 3) / Dialog.speechspeed
    else
      self.showAnswers = true
      if #self.lines == 1 and self.lines[1][1] == "" then
        self.lines[1][2]()
      end
    end
  end
end


local function lines(str)
  local l = 1
  for i = 1, #str do
    if str:sub(i, i) == "\n" then
      l = l + 1
    end
  end
  return l
end


function Dialog:draw()
  love.graphics.setFont(G.font)
  love.graphics.setColor(255,255,255)
  local name = self.origcolor[1] .. ": "
  love.graphics.print(name, 2, 132)
  love.graphics.setColor(self.origcolor[2])
  love.graphics.print(self.line, 2 + G.font:getWidth(name), 132)
  if self.showAnswers then
    local extra = (lines(self.line) - 1) * 10
    for i, l in ipairs(self.lines) do
      local y = yoffset + lineheight * i + extra

      if self.my >= y and self.my < y + lineheight then
        love.graphics.setColor(255,255,255)
      else
        love.graphics.setColor(128,128,128)
      end
      if l[1] ~= "" then
        love.graphics.print("> " .. l[1], 2, y)
      end
    end
  end
end


function Dialog:mouseover(x,y)
  self.mx = x
  self.my = y
end


function Dialog:click(x,y)
  if self.t < self.nextT then
    self.t = self.nextT
  end
  local extra = lines(self.line) * 10 - 10
  local my = math.floor((y - yoffset - extra) / lineheight)
  if my > 0 and my <= #self.lines then
    self.lines[my][2]()
  end
end


return Dialog
