local DialogSystem = Object:extend()

local Dialog = require('dialog')

local Page = Object:extend()

function Page:new(orig, origcolor, lines)
  self.orig = orig
  self.origcolor = origcolor
  self.lines = lines
end


local function cloneArray(t)
  local r = {}
  for i, v in ipairs(t) do
    r[i] = v
  end
  return r
end


function Page:setup(data, m, c)
  local lines = {}

  if self.lines[1][1] == "" then
    lines[1] = {"", function() self.lines[1][2](data) end}
  else
    for i,l in ipairs(self.lines) do
      if l[2](data) then
        lines[#lines+1] = {l[1], function() l[3](data) end}
      end
    end
  end

  if m == nil then
    m = cloneArray(self.orig)
  end
  return Dialog(m, c or self.origcolor, lines)
end


function DialogSystem:new()
  self.pages = {}
end


function DialogSystem:addPage(name, orig, origcolor, lines)
  self.pages[name] = Page(orig, origcolor, lines)
end


function DialogSystem:open(playerdata, page, skip)
  local l
  local c
  if skip then
    assert(self.dialog)
    l = self.dialog.line
    c = self.dialog.origcolor
  end

  self.dialog = self.pages[page or "start"]:setup(playerdata, l, c)
end


function DialogSystem:draw()
  if self.dialog then
    self.dialog:draw()
  end
end


function DialogSystem:update(dt)
  if self.dialog then
    self.dialog:update(dt)
  end
end


function DialogSystem:mouseover(x,y)
  if self.dialog then
    self.dialog:mouseover(x,y)
  end
end


function DialogSystem:click(x,y)
  if self.dialog then
    self.dialog:click(x,y)
  end
end


function DialogSystem:isOpen()
  return self.dialog ~= nil
end


function DialogSystem:gotoPage(name, skip)
  return function(data)
    self:open(data, name, skip)
  end
end


function DialogSystem.always()
  return true
end


function DialogSystem.tagOnce(name, action)
  return function(data)
    if not data.once then
      data.once = {}
    end

    if not data.once[name] then
      data.once[name] = true
    end

    action(data)
  end
end


function DialogSystem.once(name)
  return function(data)
    return not data.once or data.once[name] ~= true
  end
end


function DialogSystem:close()
  self.dialog = nil
end


function DialogSystem:closer()
  return function()
    self:close()
  end
end


return DialogSystem
