Person = lust.Entity:extend()

function Person:new(state, sprite, x, y, extraanim, w, h)
  Person.super.new(self, state)

  local anims = {
    {"idle", {8,8,8,8,8,8,8,8,9,9,9}, 4},
    {"walk", {0,1,2,3,4,5,6,7}, 8},
    {"punch", {10,10,10,10,11,12,13,14}, 30}
  }

  for k, v in pairs(extraanim or {}) do
    anims[k] = v
  end

  self:sprite(sprite, w or 15, h or 23, anims)


  self:play("idle")

  self:warp(x,y)

  self.offset.y = h or 23
  self.offset.x = math.floor((w or 15) / 2)
end


function Person:play(anim, noreset)
  Person.super.play(self, anim, noreset)

  if anim == "idle" and not noreset then
    self.curframe = math.random() * #self.anim.frames
  end
end


function Person:update(dt)
  Person.super.updateGfx(self, dt)

  if self.path and #self.path > 0 then
    self:play('walk', true)
    local pdx = self.path[1][1]
    local pdy = self.path[1][2]
    local dx = pdx - self.pos.x
    local dy = pdy - self.pos.y

    
    local sdx, sdy = lust.tools.scale(dx, dy, 30)

    self.scale.x = -lust.tools.sig(sdx)
    
    self:warp(sdx*dt, sdy*dt)

    local ndx = pdx - self.pos.x
    local ndy = pdy - self.pos.y

    if lust.tools.sig(ndx) ~= lust.tools.sig(sdx) or
       lust.tools.sig(ndy) ~= lust.tools.sig(sdy) or
       math.abs(ndx) < 0.1 and math.abs(ndy) < 0.1 then
      local oldp = self.path[1]
      self:warpTo(self.path[1][1], self.path[1][2])
      table.remove(self.path, 1)
      if #self.path == 0 then
        self:play('idle')
        if self.action then
          self.action()
          self.action = nil
        end
      end
    end
  end

end


function Person:draw(...)
  love.graphics.setColor(0,0,0,64)
  love.graphics.push()
  love.graphics.translate(self.pos.x, self.pos.y)
  love.graphics.scale(6,2)
  love.graphics.circle('fill', 0, 0, 1)
  love.graphics.pop()
  love.graphics.setColor(255,255,255,255)
  Person.super.draw(self, ...)
end


return Person
